﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;

namespace CleanMVCProject.Models
{
    public class RegularExpressionWorkAroundAttribute : ValidationAttribute, IClientValidatable
    {
        private string regex;

        public RegularExpressionWorkAroundAttribute(string regularExpression, string errorMessage)
        {
            this.regex = regularExpression;
            this.ErrorMessage = errorMessage;
        }

        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            // Sets up a new client rule, will have the name of kendoregularexpressionhack
            var clientValidationRule = new ModelClientValidationRule()
            {
                ErrorMessage = this.ErrorMessage,
                ValidationType = "kendoregularexpressionhack"
            };

            // Double backslash the client version of the regular expression
            string kendoRegularExpression = this.regex.Replace(@"\", @"\\");
            
            // Adds the regular expression to the attribute
            clientValidationRule.ValidationParameters.Add("regex", kendoRegularExpression);
            return new[] { clientValidationRule };
        }

        // Server side validation
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            if (value == null)
            {
                return ValidationResult.Success;
            }

            if (Regex.IsMatch(value.ToString(), this.regex))
            {
                return ValidationResult.Success;
            }

            return new ValidationResult(this.ErrorMessage);
        }

    }
}